import { Component, Input, Renderer2, ViewChild } from '@angular/core';
var styleProperties = Object.freeze([
    'direction',
    'boxSizing',
    'width',
    'height',
    'overflowX',
    'overflowY',
    'borderTopWidth',
    'borderRightWidth',
    'borderBottomWidth',
    'borderLeftWidth',
    'borderStyle',
    'paddingTop',
    'paddingRight',
    'paddingBottom',
    'paddingLeft',
    // https://developer.mozilla.org/en-US/docs/Web/CSS/font
    'fontStyle',
    'fontVariant',
    'fontWeight',
    'fontStretch',
    'fontSize',
    'fontSizeAdjust',
    'lineHeight',
    'fontFamily',
    'textAlign',
    'textTransform',
    'textIndent',
    'textDecoration',
    'letterSpacing',
    'wordSpacing',
    'tabSize',
    'MozTabSize'
]);
function indexIsInsideTag(index, tag) {
    return tag.indices.start < index && index < tag.indices.end;
}
function overlaps(tagA, tagB) {
    return (indexIsInsideTag(tagB.indices.start, tagA) ||
        indexIsInsideTag(tagB.indices.end, tagA));
}
var TextInputHighlightComponent = (function () {
    function TextInputHighlightComponent(renderer) {
        this.renderer = renderer;
        /**
         * The CSS class to add to highlighted tags
         */
        this.tagCssClass = '';
        /**
         * An array of indices of the textarea value to highlight
         */
        this.tags = [];
        /**
         * @private
         */
        this.highlightElementContainerStyle = {};
        this.textareaEventListeners = [];
    }
    /**
     * @private
     */
    TextInputHighlightComponent.prototype.ngOnChanges = function (changes) {
        if (changes.textInputElement) {
            this.textInputElementChanged();
        }
        if (changes.tags || changes.tagCssClass) {
            this.addTags();
        }
    };
    /**
     * @private
     */
    TextInputHighlightComponent.prototype.ngOnDestroy = function () {
        this.textareaEventListeners.forEach(function (unregister) { return unregister(); });
    };
    TextInputHighlightComponent.prototype.textInputElementChanged = function () {
        var _this = this;
        var elementType = this.textInputElement.tagName.toLowerCase();
        if (elementType !== 'textarea') {
            throw new Error('The angular-text-input-highlight component must be passed ' +
                'a textarea to the `textInputElement` input. Instead received a ' +
                elementType);
        }
        var computed = getComputedStyle(this.textInputElement);
        styleProperties.forEach(function (prop) {
            _this.highlightElementContainerStyle[prop] = computed[prop];
        });
        this.textareaEventListeners.forEach(function (unregister) { return unregister(); });
        this.textareaEventListeners = [
            this.renderer.listen(this.textInputElement, 'input', function () {
                _this.addTags();
            }),
            this.renderer.listen(this.textInputElement, 'scroll', function () {
                _this.highlightElement.nativeElement.scrollTop = _this.textInputElement.scrollTop;
            })
        ];
        setTimeout(function () {
            _this.addTags();
        });
    };
    TextInputHighlightComponent.prototype.addTags = function () {
        var _this = this;
        var textareaValue = this.textInputElement.value;
        var prevTags = [];
        this.tags.slice().sort(function (tagA, tagB) {
            return tagB.indices.start - tagA.indices.start;
        })
            .forEach(function (tag) {
            if (tag.indices.start > tag.indices.end) {
                throw new Error("Highlight tag with indices [" + tag.indices.start + ", " + tag.indices
                    .end + "] cannot start after it ends.");
            }
            prevTags.forEach(function (prevTag) {
                if (overlaps(prevTag, tag)) {
                    throw new Error("Highlight tag with indices [" + tag.indices.start + ", " + tag.indices
                        .end + "] overlaps with tag [" + prevTag.indices.start + ", " + prevTag
                        .indices.end + "]");
                }
            });
            // TODO - implement this as an ngFor of items that is generated in the template
            var before = textareaValue.slice(0, tag.indices.start);
            var after = textareaValue.slice(tag.indices.end);
            var expectedTagLength = tag.indices.end - tag.indices.start;
            var tagContents = textareaValue.slice(tag.indices.start, tag.indices.end);
            if (tagContents.length === expectedTagLength) {
                var cssClass = tag.cssClass || _this.tagCssClass;
                textareaValue =
                    before +
                        ("<span class=\"text-highlight-tag" + (cssClass
                            ? ' '
                            : '') + cssClass + "\">" + tagContents + "</span>") +
                        after;
                prevTags.push(tag);
            }
        });
        this.highlightedText = textareaValue + "&nbsp;";
    };
    TextInputHighlightComponent.decorators = [
        { type: Component, args: [{
                    selector: 'mwl-text-input-highlight',
                    template: "\n    <div\n      class=\"text-highlight-element\"\n      [ngStyle]=\"highlightElementContainerStyle\"\n      [innerHtml]=\"highlightedText\"\n      #highlightElement>\n    </div>\n  "
                },] },
    ];
    /** @nocollapse */
    TextInputHighlightComponent.ctorParameters = function () { return [
        { type: Renderer2, },
    ]; };
    TextInputHighlightComponent.propDecorators = {
        'tagCssClass': [{ type: Input },],
        'tags': [{ type: Input },],
        'textInputElement': [{ type: Input },],
        'highlightElement': [{ type: ViewChild, args: ['highlightElement',] },],
    };
    return TextInputHighlightComponent;
}());
export { TextInputHighlightComponent };
//# sourceMappingURL=text-input-highlight.component.js.map